@@ -41,10 +41,7 @@ along with the ASTRA Toolbox. If not, see <http://www.gnu.org/licenses/>.
 
 #include "astra/Float32ProjectionData2D.h"
 #include "astra/Float32VolumeData2D.h"
-#include "astra/SparseMatrixProjectionGeometry2D.h"
-#include "astra/FanFlatProjectionGeometry2D.h"
-#include "astra/FanFlatVecProjectionGeometry2D.h"
-#include "astra/ParallelVecProjectionGeometry2D.h"
+#include "astra/ProjectionGeometry2DFactory.h"
 
 using namespace std;
 using namespace astra;
@@ -59,8 +56,7 @@ void astra_mex_data2d_delete(int nlhs, mxArray* plhs[], int nrhs, const mxArray*
 { 
 	// step1: read input
 	if (nrhs < 2) {
-		mexErrMsgTxt("Not enough arguments.  See the help document for a detailed argument list. \n");
-		return;
+		mexErrMsgTxt("Not enough arguments. See the help document for a detailed argument list.");
 	}
 
 	// step2: delete all specified data objects
@@ -95,15 +91,14 @@ void astra_mex_data2d_create(int& nlhs, mxArray* plhs[], int& nrhs, const mxArra
 { 
 	// step1: get datatype
 	if (nrhs < 3) {
-		mexErrMsgTxt("Not enough arguments.  See the help document for a detailed argument list. \n");
-		return;
+		mexErrMsgTxt("Not enough arguments. See the help document for a detailed argument list.");
 	}
 
 	string sDataType = mexToString(prhs[1]);	
 	CFloat32Data2D* pDataObject2D = NULL;
 
 	if (nrhs >= 4 && !(mexIsScalar(prhs[3])|| mxIsDouble(prhs[3]) || mxIsLogical(prhs[3]) || mxIsSingle(prhs[3]) )) {
-		mexErrMsgTxt("Data must be single, double or logical.");
+		mexErrMsgTxt("Data type must be single, double or logical.");
 		return;
 	}
 	if (mxIsSparse(prhs[2])) {
@@ -115,86 +110,67 @@ void astra_mex_data2d_create(int& nlhs, mxArray* plhs[], int& nrhs, const mxArra
 	if (sDataType == "-vol") {
 		// Read geometry
 		if (!mxIsStruct(prhs[2])) {
-			mexErrMsgTxt("Argument 3 is not a valid MATLAB struct.\n");
+			mexErrMsgTxt("Argument 3 is not a valid MATLAB struct.");
 		}
 		
-		Config* cfg = structToConfig("VolumeGeometry", prhs[2]);
+		XMLConfig* cfg = structToConfig("VolumeGeometry", prhs[2]);
 		CVolumeGeometry2D* pGeometry = new CVolumeGeometry2D();
 		if (!pGeometry->initialize(*cfg)) {
-			mexErrMsgTxt("Geometry class not initialized. \n");
 			delete cfg;
 			delete pGeometry;
-			return;
+			mexErrMsgWithAstraLog("Geometry class could not be initialized.");
 		}
 		// If data is specified, check dimensions
 		if (nrhs >= 4 && !mexIsScalar(prhs[3])) {
 			if (pGeometry->getGridColCount() != mxGetN(prhs[3]) || pGeometry->getGridRowCount() != mxGetM(prhs[3])) {
-				mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry. \n");
 				delete cfg;
 				delete pGeometry;
-				return;
+				mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry.");
 			}
 		}
 		// Initialize data object
-		pDataObject2D = new CFloat32VolumeData2D(pGeometry);		
+		pDataObject2D = new CFloat32VolumeData2D(*pGeometry);
 		delete pGeometry;
 		delete cfg;
 	}
 	else if (sDataType == "-sino") {
 		// Read geometry
 		if (!mxIsStruct(prhs[2])) {
-			mexErrMsgTxt("Argument 3 is not a valid MATLAB struct.\n");
+			mexErrMsgTxt("Argument 3 is not a valid MATLAB struct.");
 		}
 		
-		Config* cfg = structToConfig("ProjectionGeometry", prhs[2]);
+		XMLConfig* cfg = structToConfig("ProjectionGeometry", prhs[2]);
 		// FIXME: Change how the base class is created. (This is duplicated
 		// in 'change_geometry' and Projector2D.cpp.)
 		std::string type = cfg->self.getAttribute("type");
-		CProjectionGeometry2D* pGeometry;
-		if (type == "sparse_matrix") {
-			pGeometry = new CSparseMatrixProjectionGeometry2D();
-		} else if (type == "fanflat") {
-			//CFanFlatProjectionGeometry2D* pFanFlatProjectionGeometry = new CFanFlatProjectionGeometry2D();
-			//pFanFlatProjectionGeometry->initialize(Config(node));
-			//m_pProjectionGeometry = pFanFlatProjectionGeometry;
-			pGeometry = new CFanFlatProjectionGeometry2D();	
-		} else if (type == "fanflat_vec") {
-			pGeometry = new CFanFlatVecProjectionGeometry2D();	
-		} else if (type == "parallel_vec") {
-			pGeometry = new CParallelVecProjectionGeometry2D();	
-		} else {
-			pGeometry = new CParallelProjectionGeometry2D();	
+		std::unique_ptr<CProjectionGeometry2D> pGeometry = constructProjectionGeometry2D(type);
+		if (!pGeometry) {
+			std::string message = "'" + type + "' is not a valid 2D geometry type.";
+			mexErrMsgTxt(message.c_str());
 		}
 		if (!pGeometry->initialize(*cfg)) {
-			mexErrMsgTxt("Geometry class not initialized. \n");
-			delete pGeometry;
 			delete cfg;
-			return;
+			mexErrMsgWithAstraLog("Geometry class could not be initialized.");
 		}
 		// If data is specified, check dimensions
 		if (nrhs >= 4 && !mexIsScalar(prhs[3])) {
 			if (pGeometry->getDetectorCount() != mxGetN(prhs[3]) || pGeometry->getProjectionAngleCount() != mxGetM(prhs[3])) {
-				mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry. \n");
-				delete pGeometry;
 				delete cfg;
-				return;
+				mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry.");
 			}
 		}
 		// Initialize data object
-		pDataObject2D = new CFloat32ProjectionData2D(pGeometry);
-		delete pGeometry;
+		pDataObject2D = new CFloat32ProjectionData2D(*pGeometry);
 		delete cfg;
 	}
 	else {
-		mexErrMsgTxt("Invalid datatype.  Please specify '-vol' or '-sino'. \n");
-		return;
+		mexErrMsgTxt("Invalid datatype. Please specify '-vol' or '-sino'.");
 	}
 
 	// Check initialization
 	if (!pDataObject2D->isInitialized()) {
-		mexErrMsgTxt("Couldn't initialize data object.\n");
 		delete pDataObject2D;
-		return;
+		mexErrMsgWithAstraLog("Couldn't initialize data object.");
 	}
 
 	// Store data
@@ -218,8 +194,7 @@ void astra_mex_data2d_create(int& nlhs, mxArray* plhs[], int& nrhs, const mxArra
 			const mwSize* dims = mxGetDimensions(prhs[3]);
 			// Check Data dimensions
 			if (pDataObject2D->getWidth() != mxGetN(prhs[3]) || pDataObject2D->getHeight() != mxGetM(prhs[3])) {
-				mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry. \n");
-				return;
+				mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry.");
 			}
 
 			// logical data		
@@ -265,7 +240,7 @@ void astra_mex_data2d_create(int& nlhs, mxArray* plhs[], int& nrhs, const mxArra
 	int iIndex = CData2DManager::getSingleton().store(pDataObject2D);
 
 	// step5: return data id
-	if (1 <= nlhs) {
+	if (nlhs >= 1) {
 		plhs[0] = mxCreateDoubleScalar(iIndex);
 	}
 
@@ -282,29 +257,24 @@ void astra_mex_data2d_store(int nlhs, mxArray* plhs[], int nrhs, const mxArray*
 {
 	// step1: input
 	if (nrhs < 3) {
-		mexErrMsgTxt("Not enough arguments.  See the help document for a detailed argument list. \n");
-		return;
+		mexErrMsgTxt("Not enough arguments. See the help document for a detailed argument list.");
 	}
 	if (!mxIsDouble(prhs[1])) {
-		mexErrMsgTxt("Identifier should be a scalar value. \n");
-		return;
+		mexErrMsgTxt("Identifier should be a scalar value.");
 	}
 	int iDataID = (int)(mxGetScalar(prhs[1]));
 
 	if (!(mexIsScalar(prhs[2]) || mxIsDouble(prhs[2]) || mxIsLogical(prhs[2]) || mxIsSingle(prhs[2]))) {
-		mexErrMsgTxt("Data must be single, double or logical.");
-		return;
+		mexErrMsgTxt("Data type must be single, double or logical.");
 	}
 	if (mxIsSparse(prhs[2])) {
 		mexErrMsgTxt("Data may not be sparse.");
-		return;
 	}
 
 	// step2: get data object
 	CFloat32Data2D* pDataObject = astra::CData2DManager::getSingleton().get(iDataID);
 	if (!pDataObject || !pDataObject->isInitialized()) {
-		mexErrMsgTxt("Data object not found or not initialized properly.\n");
-		return;
+		mexErrMsgTxt("Data object not found or not initialized properly.");
 	}
 	
 	// step3: insert data
@@ -317,8 +287,7 @@ void astra_mex_data2d_store(int nlhs, mxArray* plhs[], int nrhs, const mxArray*
 	} else {
 		// Check Data dimensions
 		if (pDataObject->getWidth() != mxGetN(prhs[2]) || pDataObject->getHeight() != mxGetM(prhs[2])) {
-			mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry. \n");
-			return;
+			mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry.");
 		}
 		const mwSize* dims = mxGetDimensions(prhs[2]);
 
@@ -372,31 +341,28 @@ void astra_mex_data2d_get_geometry(int nlhs, mxArray* plhs[], int nrhs, const mx
 { 
 	// parse input
 	if (nrhs < 2) {
-		mexErrMsgTxt("Not enough arguments.  See the help document for a detailed argument list. \n");
-		return;
+		mexErrMsgTxt("Not enough arguments. See the help document for a detailed argument list.");
 	}
 	if (!mxIsDouble(prhs[1])) {
-		mexErrMsgTxt("Identifier should be a scalar value. \n");
-		return;
+		mexErrMsgTxt("Identifier should be a scalar value.");
 	}
 	int iDataID = (int)(mxGetScalar(prhs[1]));
 
 	// fetch data object
 	CFloat32Data2D* pDataObject = astra::CData2DManager::getSingleton().get(iDataID);
 	if (!pDataObject || !pDataObject->isInitialized()) {
-		mexErrMsgTxt("Data object not found or not initialized properly.\n");
-		return;
+		mexErrMsgTxt("Data object not found or not initialized properly.");
 	}
 
 	// create output
 	if (1 <= nlhs) {
 		if (pDataObject->getType() == CFloat32Data2D::PROJECTION) {
 			CFloat32ProjectionData2D* pDataObject2 = dynamic_cast<CFloat32ProjectionData2D*>(pDataObject);
-			plhs[0] = configToStruct(pDataObject2->getGeometry()->getConfiguration());
+			plhs[0] = configToStruct(pDataObject2->getGeometry().getConfiguration());
 		}
 		else if (pDataObject->getType() == CFloat32Data2D::VOLUME) {
 			CFloat32VolumeData2D* pDataObject2 = dynamic_cast<CFloat32VolumeData2D*>(pDataObject);
-			plhs[0] = configToStruct(pDataObject2->getGeometry()->getConfiguration());
+			plhs[0] = configToStruct(pDataObject2->getGeometry().getConfiguration());
 		}
 	}
 }
@@ -412,20 +378,17 @@ void astra_mex_data2d_change_geometry(int nlhs, mxArray* plhs[], int nrhs, const
 { 
 	// step1: check input
 	if (nrhs < 3) {
-		mexErrMsgTxt("Not enough arguments.  See the help document for a detailed argument list. \n");
-		return;
+		mexErrMsgTxt("Not enough arguments. See the help document for a detailed argument list.");
 	}
 	if (!mxIsDouble(prhs[1])) {
-		mexErrMsgTxt("Identifier should be a scalar value. \n");
-		return;
+		mexErrMsgTxt("Identifier should be a scalar value.");
 	}
 
 	// step2: get data object
 	int iDataID = (int)(mxGetScalar(prhs[1]));
 	CFloat32Data2D* pDataObject = astra::CData2DManager::getSingleton().get(iDataID);
 	if (!pDataObject || !pDataObject->isInitialized()) {
-		mexErrMsgTxt("Data object not found or not initialized properly.\n");
-		return;
+		mexErrMsgTxt("Data object not found or not initialized properly.");
 	}
 
 	CFloat32ProjectionData2D* pSinogram = dynamic_cast<CFloat32ProjectionData2D*>(pDataObject);
@@ -435,44 +398,30 @@ void astra_mex_data2d_change_geometry(int nlhs, mxArray* plhs[], int nrhs, const
 
 		// Read geometry
 		if (!mxIsStruct(prhs[2])) {
-			mexErrMsgTxt("Argument 3 is not a valid MATLAB struct.\n");
+			mexErrMsgTxt("Argument 3 is not a valid MATLAB struct.");
 		}
-		Config* cfg = structToConfig("ProjectionGeometry2D", prhs[2]);
+		XMLConfig* cfg = structToConfig("ProjectionGeometry2D", prhs[2]);
 		// FIXME: Change how the base class is created. (This is duplicated
 		// in 'create' and Projector2D.cpp.)
 		std::string type = cfg->self.getAttribute("type");
-		CProjectionGeometry2D* pGeometry;
-		if (type == "sparse_matrix") {
-			pGeometry = new CSparseMatrixProjectionGeometry2D();
-		} else if (type == "fanflat") {
-			//CFanFlatProjectionGeometry2D* pFanFlatProjectionGeometry = new CFanFlatProjectionGeometry2D();
-			//pFanFlatProjectionGeometry->initialize(Config(node));
-			//m_pProjectionGeometry = pFanFlatProjectionGeometry;
-			pGeometry = new CFanFlatProjectionGeometry2D();	
-		} else if (type == "fanflat_vec") {
-			pGeometry = new CFanFlatVecProjectionGeometry2D();	
-		} else if (type == "parallel_vec") {
-			pGeometry = new CParallelVecProjectionGeometry2D();	
-		} else {
-			pGeometry = new CParallelProjectionGeometry2D();	
+		std::unique_ptr<CProjectionGeometry2D> pGeometry = constructProjectionGeometry2D(type);
+		if (!pGeometry) {
+			delete cfg;
+			std::string message = "'" + type + "' is not a valid 2D geometry type.";
+			mexErrMsgTxt(message.c_str());
 		}
 		if (!pGeometry->initialize(*cfg)) {
-			mexErrMsgTxt("Geometry class not initialized. \n");
-			delete pGeometry;
 			delete cfg;
-			return;
+			mexErrMsgWithAstraLog("Geometry could not be initialized.");
 		}
 		// If data is specified, check dimensions
 		if (pGeometry->getDetectorCount() != pSinogram->getDetectorCount() || pGeometry->getProjectionAngleCount() != pSinogram->getAngleCount()) {
-			mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry. \n");
-			delete pGeometry;
 			delete cfg;
-			return;
+			mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry.");
 		}
 
 		// If ok, change geometry
-		pSinogram->changeGeometry(pGeometry);
-		delete pGeometry;
+		pSinogram->changeGeometry(*pGeometry);
 		delete cfg;
 
 		return;
@@ -485,33 +434,32 @@ void astra_mex_data2d_change_geometry(int nlhs, mxArray* plhs[], int nrhs, const
 
 		// Read geometry
 		if (!mxIsStruct(prhs[2])) {
-			mexErrMsgTxt("Argument 3 is not a valid MATLAB struct.\n");
+			mexErrMsgTxt("Argument 3 is not a valid MATLAB struct.");
 		}
-		Config* cfg = structToConfig("VolumeGeometry2D", prhs[2]);
+		XMLConfig* cfg = structToConfig("VolumeGeometry2D", prhs[2]);
 		CVolumeGeometry2D* pGeometry = new CVolumeGeometry2D();
 		if (!pGeometry->initialize(*cfg)) {
-			mexErrMsgTxt("Geometry class not initialized. \n");
 			delete cfg;
 			delete pGeometry;
-			return;
+			mexErrMsgWithAstraLog("Geometry class could not be initialized.");
 		}
 		// If data is specified, check dimensions
 		if (pGeometry->getGridColCount() != pVolume->getWidth() || pGeometry->getGridRowCount() != pVolume->getHeight()) {
-			mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry. \n");
 			delete cfg;
 			delete pGeometry;
-			return;
+			mexErrMsgTxt("The dimensions of the data do not match those specified in the geometry.");
 		}
 
 		// If ok, change geometry
-		pVolume->changeGeometry(pGeometry);
+		pVolume->changeGeometry(*pGeometry);
 		delete cfg;
 		delete pGeometry;
 
+		return;
 	}
 
-	mexErrMsgTxt("Data object not found or not initialized properly.\n");
-	return;
+	// Neither sinogram nor volume object
+	mexErrMsgTxt("Data object not found or not initialized properly.");
 }
 
 //-----------------------------------------------------------------------------------------
@@ -525,20 +473,17 @@ void astra_mex_data2d_get(int nlhs, mxArray* plhs[], int nrhs, const mxArray* pr
 { 
 	// step1: check input
 	if (nrhs < 2) {
-		mexErrMsgTxt("Not enough arguments.  See the help document for a detailed argument list. \n");
-		return;
+		mexErrMsgTxt("Not enough arguments. See the help document for a detailed argument list.");
 	}
 	if (!mxIsDouble(prhs[1])) {
-		mexErrMsgTxt("Identifier should be a scalar value. \n");
-		return;
+		mexErrMsgTxt("Identifier should be a scalar value.");
 	}
 
 	// step2: get data object
 	int iDataID = (int)(mxGetScalar(prhs[1]));
 	CFloat32Data2D* pDataObject = astra::CData2DManager::getSingleton().get(iDataID);
 	if (!pDataObject || !pDataObject->isInitialized()) {
-		mexErrMsgTxt("Data object not found or not initialized properly.\n");
-		return;
+		mexErrMsgTxt("Data object not found or not initialized properly.");
 	}
 
 	// create output
@@ -570,20 +515,17 @@ void astra_mex_data2d_get_single(int nlhs, mxArray* plhs[], int nrhs, const mxAr
 { 
 	// step1: check input
 	if (nrhs < 2) {
-		mexErrMsgTxt("Not enough arguments.  See the help document for a detailed argument list. \n");
-		return;
+		mexErrMsgTxt("Not enough arguments. See the help document for a detailed argument list.");
 	}
 	if (!mxIsDouble(prhs[1])) {
-		mexErrMsgTxt("Identifier should be a scalar value. \n");
-		return;
+		mexErrMsgTxt("Identifier should be a scalar value.");
 	}
 
 	// step2: get data object
 	int iDataID = (int)(mxGetScalar(prhs[1]));
 	CFloat32Data2D* pDataObject = astra::CData2DManager::getSingleton().get(iDataID);
 	if (!pDataObject || !pDataObject->isInitialized()) {
-		mexErrMsgTxt("Data object not found or not initialized properly.\n");
-		return;
+		mexErrMsgTxt("Data object not found or not initialized properly.");
 	}
 
 	// create output
