@@ -1,7 +1,21 @@
+class GILHelper {
+public:
+    GILHelper() {
+        state = PyGILState_Ensure();
+    }
+    ~GILHelper() {
+        PyGILState_Release(state);
+    }
+private:
+    PyGILState_STATE state;
+};
+
 class CFloat32CustomPython : public astra::CFloat32CustomMemory {
 public:
-    CFloat32CustomPython(PyObject * arrIn)
+    CFloat32CustomPython(PyArrayObject * arrIn)
     {
+        GILHelper gil; // hold GIL during this function
+
         arr = arrIn;
         // Set pointer to numpy data pointer
         m_fPtr = (float *)PyArray_DATA(arr);
@@ -9,9 +23,11 @@ public:
         Py_INCREF(arr);
     }
     virtual ~CFloat32CustomPython() {
+        GILHelper gil; // hold GIL during this function
+
         // Decrease reference count since ASTRA object is destroyed
         Py_DECREF(arr);
     }
 private:
-    PyObject* arr;
-};
\ No newline at end of file
+    PyArrayObject* arr;
+};
