@@ -8,12 +8,15 @@
 % Website: http://www.astra-toolbox.com/
 % -----------------------------------------------------------------------
 
+% Switch to GPU #1. The default is GPU #0.
+astra_set_gpu_index(1);
+
 vol_geom = astra_create_vol_geom(256, 256);
 proj_geom = astra_create_proj_geom('parallel', 1.0, 384, linspace2(0,pi,180));
 P = phantom(256);
 
-% Create a sinogram from a phantom, using GPU #1. (The default is #0)
-[sinogram_id, sinogram] = astra_create_sino_gpu(P, proj_geom, vol_geom, 1);
+% Create a sinogram from a phantom.
+[sinogram_id, sinogram] = astra_create_sino_gpu(P, proj_geom, vol_geom);
 
 
 % Set up the parameters for a reconstruction algorithm using the GPU
@@ -22,9 +25,6 @@ cfg = astra_struct('SIRT_CUDA');
 cfg.ReconstructionDataId = rec_id;
 cfg.ProjectionDataId = sinogram_id;
 
-% Use GPU #1 for the reconstruction. (The default is #0.)
-cfg.option.GPUindex = 1;
-
 % Run 150 iterations of the algorithm
 alg_id = astra_mex_algorithm('create', cfg);
 astra_mex_algorithm('iterate', alg_id, 150);
